#! /usr/bin/env python

import argparse,clipboard,time
import logging,getpass
import os
import ee
import subprocess
os.chdir(os.path.dirname(os.path.realpath(__file__)))
from ee import oauth
from batch_copy import copy
from batch_remover import delete
from batch_uploader import upload
from config import setup_logging
from batch_mover import mover
from tabup import tabup
from taskrep import genreport
from acl_changer import access
from ee_ls import lst
from assetsizes import assetsize
from ee_report import ee_report
from ee_del_meta import delprop
from zipfiles import zipshape


suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
def humansize(nbytes):
    i = 0
    while nbytes >= 1024 and i < len(suffixes)-1:
        nbytes /= 1024.
        i += 1
    f = ('%.2f' % nbytes).rstrip('0').rstrip('.')
    return '%s %s' % (f, suffixes[i])

def cancel_all_running_tasks():
    logging.info('Attempting to cancel all running tasks')
    running_tasks = [task for task in ee.data.getTaskList() if task['state'] == 'RUNNING']
    for task in running_tasks:
        ee.data.cancelTask(task['id'])
    logging.info('Cancel all request completed')

def cancel_all_running_tasks_from_parser(args):
    cancel_all_running_tasks()

def delete_collection_from_parser(args):
    delete(args.id)

def zipshape_from_parser(args):
    zipshape(directory=args.input,export=args.output)

def upload_from_parser(args):
    upload(user=args.user,
           source_path=args.source,
           destination_path=args.dest,
           metadata_path=args.metadata,
           multipart_upload=args.large,
           nodata_value=args.nodata,
           bucket_name=args.bucket,
           band_names=args.bands)
def tabup_from_parser(args):
    tabup(user=args.user,
           source_path=args.source,
           destination_path=args.dest)

def _comma_separated_strings(string):
  """Parses an input consisting of comma-separated strings.
     Slightly modified version of: https://pypkg.com/pypi/earthengine-api/f/ee/cli/commands.py
  """
  error_msg = 'Argument should be a comma-separated list of alphanumeric strings (no spaces or other' \
              'special characters): {}'
  values = string.split(',')
  for name in values:
      if not name.isalnum():
          raise argparse.ArgumentTypeError(error_msg.format(string))
  return values

def quota():
    quota=ee.data.getAssetRootQuota(ee.data.getAssetRoots()[0]['id'])
    print('')
    print("Total Quota: "+str(humansize(quota['asset_size']['limit'])))
    print("Used Quota: "+str(humansize(quota['asset_size']['usage'])))

def quota_from_parser(args):
    quota()

def ee_report_from_parser(args):
    ee_report(output=args.outfile)

def mover_from_parser(args):
	mover(collection_path=args.assetpath,final_path=args.finalpath)
def copy_from_parser(args):
	copy(collection_path=args.initial,final_path=args.final)
def access_from_parser(args):
	access(collection_path=args.asset,user=args.user,role=args.role)
def delete_metadata_from_parser(args):
    delprop(collection_path=args.asset,property=args.property)

def tasks():
    statuses=ee.data.getTaskList()
    st=[]
    for status in statuses:
        st.append(status['state'])
    print("Tasks Running: "+str(st.count('RUNNING')))
    print("Tasks Ready: "+str(st.count('READY')))
    print("Tasks Completed: "+str(st.count('COMPLETED')))
    print("Tasks Failed: "+str(st.count('FAILED')))
    print("Tasks Cancelled: "+str(st.count('CANCELLED')))

def tasks_from_parser(args):
    tasks()

def ee_auth_entry():
    auth_url = ee.oauth.get_authorization_url()
    clipboard.copy(auth_url)
    print("Authentication link copied: Go to browser and click paste")
    time.sleep(10)
    print("Enter your GEE API Token")
    password=str(getpass.getpass())
    auth_code=str(password)
    token = ee.oauth.request_token(auth_code)
    ee.oauth.write_token(token)
    print('\nSuccessfully saved authorization token.')
def ee_user_from_parser(args):
    ee_auth_entry()
def create_from_parser(args):
    typ=str(args.typ)
    ee_path=str(args.path)
    os.system("earthengine create "+typ+" "+ee_path)

def genreport_from_parser(args):
    genreport(report=args.r)
def assetsize_from_parser(args):
    assetsize(asset=args.asset)
def lst_from_parser(args):
    lst(location=args.location,typ=args.typ,items=args.items,output=args.output)

def main(args=None):
    setup_logging()
    parser = argparse.ArgumentParser(description='Google Earth Engine Batch Asset Manager with Addons')
    parser.add_argument('-s', '--service-account', help='Google Earth Engine service account.', required=False)
    parser.add_argument('-k', '--private-key', help='Google Earth Engine private key file.', required=False)

    subparsers = parser.add_subparsers()
    parser_ee_user=subparsers.add_parser('ee_user',help='Allows you to associate/change GEE account to system')
    parser_ee_user.set_defaults(func=ee_user_from_parser)

    parser_quota = subparsers.add_parser('quota', help='Print Earth Engine total quota and used quota')
    parser_quota.set_defaults(func=quota_from_parser)

    parser_create = subparsers.add_parser('create',help='Allows the user to create an asset collection or folder in Google Earth Engine')
    parser_create.add_argument('--typ', help='Specify type: collection or folder', required=True)
    parser_create.add_argument('--path', help='This is the path for the earth engine asset to be created full path is needsed eg: users/johndoe/collection', required=True)
    parser_create.set_defaults(func=create_from_parser)

    parser_zipshape = subparsers.add_parser('zipshape', help='Zips all shapefiles and subsidary files into individual zip files')
    required_named = parser_zipshape.add_argument_group('Required named arguments.')
    required_named.add_argument('--input', help='Path to the input directory with all shape files', required=True)
    required_named.add_argument('--output', help='Destination folder Full path where shp, shx, prj and dbf files if present in input will be zipped and stored', required=True)
    parser_zipshape.set_defaults(func=zipshape_from_parser)

    parser_upload = subparsers.add_parser('upload', help='Batch Asset Uploader upload images to collection')
    required_named = parser_upload.add_argument_group('Required named arguments.')
    required_named.add_argument('--source', help='Path to the directory with images for upload.', required=True)
    required_named.add_argument('--dest', help='Destination. Full path for upload to Google Earth Engine, e.g. users/pinkiepie/myponycollection', required=True)
    optional_named = parser_upload.add_argument_group('Optional named arguments')
    optional_named.add_argument('-m', '--metadata', help='Path to CSV with metadata.')
    optional_named.add_argument('--large', action='store_true', help='(Advanced) Use multipart upload. Might help if upload of large '
                                                                     'files is failing on some systems. Might cause other issues.')
    optional_named.add_argument('--nodata', type=int, help='The value to burn into the raster as NoData (missing data)')
    optional_named.add_argument('--bands', type=_comma_separated_strings, help='Comma-separated list of names to use for the image bands. Spaces'
                                                                               'or other special characters are not allowed.')

    required_named.add_argument('-u', '--user', help='Google account name (gmail address).')
    optional_named.add_argument('-b', '--bucket', help='Google Cloud Storage bucket name.')

    parser_upload.set_defaults(func=upload_from_parser)

    parser_tabup = subparsers.add_parser('tabup', help='Batch Table Uploader upload the shapefiles you zipped earlier.')
    required_named = parser_tabup.add_argument_group('Required named arguments.')
    required_named.add_argument('--source', help='Path to the directory with zipped folder for upload.', required=True)
    required_named.add_argument('--dest', help='Destination. Full path for upload to Google Earth Engine, e.g. users/pinkiepie/myponycollection', required=True)
    required_named.add_argument('-u', '--user', help='Google account name (gmail address).')
    parser_tabup.set_defaults(func=tabup_from_parser)

    parser_lst = subparsers.add_parser('lst',help='List assets in a folder/collection or write as text file')
    required_named = parser_lst.add_argument_group('Required named arguments.')
    required_named.add_argument('--location', help='This it the location of your folder/collection', required=True)
    required_named.add_argument('--typ', help='Whether you want the list to be printed or output as text[print/report]', required=True)
    optional_named = parser_lst.add_argument_group('Optional named arguments')
    optional_named.add_argument('--items', help="Number of items to list")
    optional_named.add_argument('--output',help="Folder location for report to be exported")
    parser_lst.set_defaults(func=lst_from_parser)

    parser_ee_report = subparsers.add_parser('ee_report',help='Prints a detailed report of all Earth Engine Assets includes Asset Type, Path,Number of Assets,size(MB),unit,owner,readers,writers')
    parser_ee_report.add_argument('--outfile', help='This it the location of your report csv file ', required=True)
    parser_ee_report.set_defaults(func=ee_report_from_parser)

    parser_assetsize = subparsers.add_parser('assetsize',help='Prints collection size in Human Readable form & Number of assets')
    parser_assetsize.add_argument('--asset', help='Earth Engine Asset for which to get size properties', required=True)
    parser_assetsize.set_defaults(func=assetsize_from_parser)

    parser_tasks=subparsers.add_parser('tasks',help='Queries current task status [completed,running,ready,failed,cancelled]')
    parser_tasks.set_defaults(func=tasks_from_parser)

    parser_genreport=subparsers.add_parser('taskreport',help='Create a report of all tasks and exports to a CSV file')
    parser_genreport.add_argument('--r',help='Path to csv report file')
    parser_genreport.set_defaults(func=genreport_from_parser)


    parser_delete = subparsers.add_parser('delete', help='Deletes collection and all items inside. Supports Unix-like wildcards.')
    parser_delete.add_argument('id', help='Full path to asset for deletion. Recursively removes all folders, collections and images.')
    parser_delete.set_defaults(func=delete_collection_from_parser)

    parser_mover=subparsers.add_parser('mover',help='Moves all assets from one collection to another')
    parser_mover.add_argument('--assetpath',help='Existing path of assets')
    parser_mover.add_argument('--finalpath',help='New path for assets')
    parser_mover.set_defaults(func=mover_from_parser)

    parser_copy=subparsers.add_parser('copy',help='Copies all assets from one collection to another: Including copying from other users if you have read permission to their assets')
    parser_copy.add_argument('--initial',help='Existing path of assets')
    parser_copy.add_argument('--final',help='New path for assets')
    parser_copy.set_defaults(func=copy_from_parser)

    parser_access = subparsers.add_parser('access',help='Sets Permissions for items in folder')
    parser_access.add_argument('--asset', help='This is the path to the earth engine asset whose permission you are changing folder/collection/image', required=True)
    parser_access.add_argument('--user', help='Full email address of the user, try using "AllUsers" to make it public', required=True, default=False)
    parser_access.add_argument('--role', help='Choose between reader, writer or delete', required=True)
    parser_access.set_defaults(func=access_from_parser)

    parser_delete_metadata = subparsers.add_parser('delete_metadata',help='Use with caution: delete any metadata from collection or image')
    parser_delete_metadata.add_argument('--asset', help='This is the path to the earth engine asset whose permission you are changing collection/image', required=True)
    parser_delete_metadata.add_argument('--property', help='Metadata name that you want to delete', required=True, default=False)
    parser_delete_metadata.set_defaults(func=delete_metadata_from_parser)

    parser_cancel = subparsers.add_parser('cancel', help='Cancel all running tasks')
    parser_cancel.set_defaults(func=cancel_all_running_tasks_from_parser)

    args = parser.parse_args()

    #ee.Initialize()
    args.func(args)

if __name__ == '__main__':
    main()



from __future__ import print_function
import ee,json,csv,subprocess
ee.Initialize()
def ee_report(output):
    with open(output,"wb") as csvfile:
        writer=csv.DictWriter(csvfile,fieldnames=["type", "path","No of Assets","size","unit","owner","readers","writers"], delimiter=',')
        writer.writeheader()
    a=subprocess.check_output("earthengine ls",shell=True)
    b=subprocess.check_output('earthengine ls -l -r '+a,shell=True)
    try:
        for item in b.split('\n'):
            a=item.replace("[","").replace("]","").split()
            header=a[0]
            tail=a[1]
            if header=="ImageCollection":
                collc=ee.ImageCollection(tail)
                own=json.dumps(json.loads(subprocess.check_output('earthengine acl get '+tail,shell=True)), ensure_ascii=False)
                #print(own)
                o=str(own).split('"owners": [')[1].split("]")[0].replace('"','')
                r=str(own).split('"readers": [')[1].split("]")[0].replace('"','')
                w=str(own).split('"writers": [')[1].split("]")[0].replace('"','')
                print("Processing Image Collection "+str(tail))
                ast=collc.size().getInfo()
                size=collc.reduceColumns(ee.Reducer.sum(), ['system:asset_size']).get('sum').getInfo()
                sz=float(size)/1000000
                unit="MB"
                with open(output,"a") as csvfile:
                    writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                    writer.writerow([header,tail,ast,format(sz,'.2f'),unit,o,r,w])
                csvfile.close()
            elif header=="Image":
                collc=ee.Image(tail)
                own=json.dumps(json.loads(subprocess.check_output('earthengine acl get '+tail,shell=True)), ensure_ascii=False)
                o=str(own).split('"owners": [')[1].split("]")[0].replace('"','')
                r=str(own).split('"readers": [')[1].split("]")[0].replace('"','')
                w=str(own).split('"writers": [')[1].split("]")[0].replace('"','')
                print("Processing Image "+str(tail))
                size=collc.get('system:asset_size').getInfo()
                ast="1"
                sz=float(size)/1000000
                unit="MB"
                with open(output,"a") as csvfile:
                    writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                    writer.writerow([header,tail,ast,format(sz,'.2f'),unit,o,r,w])
                csvfile.close()
            elif header=="Table":
                collc=ee.FeatureCollection(tail)
                own=json.dumps(json.loads(subprocess.check_output('earthengine acl get '+tail,shell=True)), ensure_ascii=False)
                o=str(own).split('"owners": [')[1].split("]")[0].replace('"','')
                r=str(own).split('"readers": [')[1].split("]")[0].replace('"','')
                w=str(own).split('"writers": [')[1].split("]")[0].replace('"','')
                #print(o,r,w)
                print("Processing Table "+str(tail))
                ast="1"
                size=collc.get('system:asset_size').getInfo()
                sz=float(size)/1000000
                unit="MB"
                with open(output,"a") as csvfile:
                    writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                    writer.writerow([header,tail,ast,format(sz,'.2f'),unit,o,r,w])
                csvfile.close()
            elif header=="Folder": ##Folders are not added to the list but are print
                ast=int(b.count(tail))-1
                print("Folder "+str(tail) +" has "+str(ast)+" assets")
    except Exception:
        return "worked"
if __name__ == '__main__':
    ee_report(None)



import ee
import os


ee.Initialize()

def delprop(collection_path,property):
    header=ee.data.getInfo(collection_path)['type']
    if header=="Image":
        lst=[]
        lst.append(collection_path)
        assets_names=lst
    if header=="ImageCollection":
        assets_list = ee.data.getList(params={'id': collection_path})
        assets_names = [os.path.basename(asset['id']) for asset in assets_list]
    print('Changing permission for total of '+str(len(assets_names))+'.....')
    for count,items in enumerate(assets_names):
        if header=='ImageCollection':
            nullgrid={property:None}
            init=collection_path+'/'+items
            try:
                print("Processing "+str(count+1)+' of '+str(len(assets_names)))
                ee.data.setAssetProperties(init, nullgrid)
            except Exception as e:
                print("Could not run "+str(count+1)+' of '+str(len(assets_names)))
                print(e)
        else:
            init=items
            nullgrid={property:None}
            try:
                print("Processing "+str(count+1)+' of '+str(len(assets_names)))
                ee.data.setAssetProperties(init, nullgrid)
            except Exception as e:
                print("Could not run "+str(count+1)+' of '+str(len(assets_names)))
                print(e)

#delprop(collection_path='users/samapriya/LA_ASSET_EXP/LS_UNMX_OTSU_MEDOID',property='gridded')
#ee.data.setAssetProperties(args.asset_id, properties)



import ast
import collections
import csv
import logging
import re

ValidationResult = collections.namedtuple('ValidationResult', ['success', 'keys'])


class IllegalPropertyName(Exception):
    pass


def validate_metadata_from_csv(path):
    """
    Check if metadata is ok
    :param path:
    :return: true / false
    """
    all_keys = []

    with open(path, mode='r') as metadata_file:
        logging.info('Running metatdata validator for %s', path)
        success = True
        reader = csv.reader(metadata_file)
        header = next(reader)

        if not properties_allowed(properties=header, validator=allowed_property_key):
            raise IllegalPropertyName('The header has illegal name.')

        for row in reader:
            all_keys.append(row[0])
            if not properties_allowed(properties=row, validator=allowed_property_value):
                success = False

        logging.info('Validation successful') if success else logging.error('Validation failed')

        return ValidationResult(success=success, keys=all_keys)


def load_metadata_from_csv(path):
    """
    Grabs properties from the give csv file. The csv should be organised as follows:
    filename (without extension), property1, property2, ...

    Example:
    id_no,class,category,binomial
    my_file_1,GASTROPODA,EN,Aaadonta constricta
    my_file_2,GASTROPODA,CR,Aaadonta irregularis

    The corresponding files are my_file_1.tif and my_file_2.tif.

    The program will turn the above into a json object:

    { id_no: my_file_1, class: GASTROPODA, category: EN, binomial: Aaadonta constricta},
    { id_no: my_file_2, class: GASTROPODA, category: CR, binomial: Aaadonta irregularis}

    :param path to csv:
    :return: dictionary of dictionaries
    """
    with open(path, mode='r') as metadata_file:
        reader = csv.reader(metadata_file)
        header = next(reader)

        if not properties_allowed(properties=header, validator=allowed_property_key):
            raise IllegalPropertyName()

        metadata = {}

        for row in reader:
#            if properties_allowed(properties=row, validator=allowed_property_value):
            values = []
            for item in row:
                try:
                    values.append(ast.literal_eval(item))
                except (ValueError, SyntaxError) as e:
                    values.append(item)
            metadata[row[0]] = dict(zip(header, values))

        return metadata


def properties_allowed(properties, validator):
    return all(validator(prop) for prop in properties)


def allowed_property_key(prop):
    google_special_properties = ('system:description',
                                 'system:provider_url',
                                 'system:tags',
                                 'system:time_end',
                                 'system:time_start',
                                 'system:title')

    if prop in google_special_properties or re.match("^[A-Za-z0-9_]+$", prop):
        return True
    else:
        logging.warning('Property name %s is invalid. Special properties [system:description, system:provider_url, '
                        'system:tags, system:time_end, system:time_start, system:title] are allowed; other property '
                        'keys must contain only letters, digits and underscores.')
        return False



import fnmatch
import logging
import sys

import ee


def delete(asset_path):
    root = asset_path[:asset_path.rfind('/')]
    all_assets_names = [e['id'] for e in ee.data.getList({'id': root})]
    filtered_names = fnmatch.filter(all_assets_names, asset_path)
    if not filtered_names:
        logging.warning('Nothing to remove. Exiting.')
        sys.exit(1)
    else:
        for path in filtered_names:
            __delete_recursive(path)
            logging.info('Collection %s removed', path)


def __delete_recursive(asset_path):
    info = ee.data.getInfo(asset_path)
    if not info:
        logging.warning('Nothing to delete.')
        sys.exit(1)
    elif info['type'] == 'Image':
        pass
    elif info['type'] == 'Folder':
        items_in_destination = ee.data.getList({'id': asset_path})
        for item in items_in_destination:
            logging.info('Removing items in %s folder', item['id'])
            delete(item['id'])
    else:
        items_in_destination = ee.data.getList({'id': asset_path})
        for item in items_in_destination:
            ee.data.deleteAsset(item['id'])
    ee.data.deleteAsset(asset_path)



import ee
import datetime
import csv

ee.Initialize()

def genreport(report):
    with open(report,'wb') as failed:
        writer=csv.DictWriter(failed,fieldnames=["Task ID","Task Type", "Task Description","Creation","Start","End","Time to Start","Time to End","Output State"],delimiter=',')
        writer.writeheader()
    status=ee.data.getTaskList()
    for items in status:
        ttype=items['task_type']
        tdesc=items['description']
        tstate=items['state']
        tid=items['id']
        try:
            tcreate=datetime.datetime.fromtimestamp(items['creation_timestamp_ms']/1000).strftime('%c')
            tstart=datetime.datetime.fromtimestamp(items['start_timestamp_ms']/1000).strftime('%c')
            tupdate=datetime.datetime.fromtimestamp(items['update_timestamp_ms']/1000).strftime('%c')
            tdiffstart=items['start_timestamp_ms']/1000-items['creation_timestamp_ms']/1000
            tdiffend=items['update_timestamp_ms']/1000-items['start_timestamp_ms']/1000
            #print(ttype,tdesc,tstate,tid,tcreate,tstart,tupdate,tdiffstart,tdiffend)
            with open(report,'a') as tasks:
                writer=csv.writer(tasks,delimiter=',',lineterminator='\n')
                writer.writerow([tid,ttype,tdesc,str(tcreate),str(tstart),str(tupdate),str(tdiffstart),str(tdiffend),tstate])
        except Exception as e:
            pass
    print('Report exported to '+str(report))
#genreport(report=r'C:\planet_demo\taskrep.csv')



import ee
import os
import json

##initialize earth engine
ee.Initialize()

##request type of asset, asset path and user to give permission
def access(collection_path,user,role):
    header=ee.data.getInfo(collection_path)['type']
    if header=="ImageCollection":
        lst=[]
        lst.append(collection_path)
        assets_names=lst
    elif header=="Image":
        lst=[]
        lst.append(collection_path)
        assets_names=lst
    elif header=="Table":
        lst=[]
        lst.append(collection_path)
        assets_names=lst
    else:
        assets_list = ee.data.getList(params={'id': collection_path})
        assets_names = [os.path.basename(asset['id']) for asset in assets_list]
    print('Changing permission for total of '+str(len(assets_names))+'.....')
    for count,items in enumerate(assets_names):
        if header=='Folder':
            init=collection_path+'/'+items
            print('Working on item '+str(init))
        else:
            init=items
            print('Working on item '+str(init))
        acl = ee.data.getAssetAcl(init)
        if role=='reader':
            if not user in acl['readers']:
                baselist=acl['readers']
                baselist.append(user)
                acl['readers']=baselist
                acl['owners']=[]
                try:
                    ee.data.setAssetAcl(init, json.dumps(acl))
                except Exception as e:
                    print(e)
            else:
                print('user already has read access to this asset:SKIPPING')
        if role=='writer':
            if not user in acl['writers']:
                baselist=acl['writers']
                baselist.append(user)
                acl['readers']=baselist
                acl['owners']=[]
                try:
                    ee.data.setAssetAcl(init, json.dumps(acl))
                except Exception as e:
                    print(e)
            else:
                print('user already has write access to this asset:SKIPPING')
        if role=='delete':
            if not user in acl['readers']:
                print('user does not have permission:SKIPPING')
            else:
                baselist=acl['readers']
                baselist.remove(user)
                acl['readers']=baselist
                acl['owners']=[]
                try:
                    ee.data.setAssetAcl(init, json.dumps(acl))
                except Exception as e:
                    print(e)



import ee
import subprocess
import csv
import os

##initialize earth engine
ee.Initialize()

def lst(location, typ, items=None,output=None):
    if items > 0 and typ =='print':
        b=subprocess.check_output("earthengine ls -l -r "+'"'+location+'"'+" --max_items "+str(items),shell=True)
        try:
            for item in b.split('\n'):
                a=item.replace("[","").replace("]","").split()
                header=a[0]
                tail=a[1]
                if header=="ImageCollection":
                    collc=ee.ImageCollection(tail)
                    ast=collc.size().getInfo()
                    size=collc.reduceColumns(ee.Reducer.sum(), ['system:asset_size']).get('sum').getInfo()
                    sz=float(size)/1000000
                    print("Image Collection "+str(tail)+" has "+str(ast)+" images a total of "+format(sz,'.2f')+" MB")
                elif header=="Image":
                    collc=ee.Image(tail)
                    size=collc.get('system:asset_size').getInfo()
                    sz=float(size)/1000000
                    print("Image "+str(tail)+" is "+format(sz,'.2f')+" MB")
                elif header=="Table":
                    collc=ee.FeatureCollection(tail)
                    size=collc.get('system:asset_size').getInfo()
                    sz=float(size)/1000000
                    print("Table "+str(tail)+" is "+format(sz,'.2f')+" MB")
                elif header=="Folder": ##Folders are not added to the list but are print
                    ast=int(b.count(tail))-1
                    print("Folder "+str(tail) +" has "+format(sz,'.2f')+" assets")
        except Exception:
            return "worked"
    elif items is None and typ =='print':
        b=subprocess.check_output("earthengine ls -l -r "+'"'+location+'"',shell=True)
        try:
            for item in b.split('\n'):
                a=item.replace("[","").replace("]","").split()
                header=a[0]
                tail=a[1]
                if header=="ImageCollection":
                    collc=ee.ImageCollection(tail)
                    ast=collc.size().getInfo()
                    size=collc.reduceColumns(ee.Reducer.sum(), ['system:asset_size']).get('sum').getInfo()
                    sz=float(size)/1000000
                    print("Image Collection "+str(tail)+" has "+str(ast)+" images a total of "+format(sz,'.2f')+" MB")
                elif header=="Image":
                    collc=ee.Image(tail)
                    size=collc.get('system:asset_size').getInfo()
                    sz=float(size)/1000000
                    print("Image "+str(tail)+" is "+format(sz,'.2f')+" MB")
                elif header=="Table":
                    collc=ee.FeatureCollection(tail)
                    size=collc.get('system:asset_size').getInfo()
                    sz=float(size)/1000000
                    print("Table "+str(tail)+" is "+format(sz,'.2f')+" MB")
                elif header=="Folder": ##Folders are not added to the list but are print
                    ast=int(b.count(tail))-1
                    print("Folder "+str(tail) +" has "+format(sz,'.2f')+" assets")
        except Exception:
            return "worked"
    elif items is None and typ =='report':
        with open(output,"wb") as csvfile:
            writer=csv.DictWriter(csvfile,fieldnames=["type", "path","No of Assets","size","unit"], delimiter=',')
            writer.writeheader()
            b=subprocess.check_output("earthengine ls -l -r "+'"'+location+'"',shell=True)
        try:
            for item in b.split('\n'):
                a=item.replace("[","").replace("]","").split()
                header=a[0]
                tail=a[1]
                if header=="ImageCollection":
                    collc=ee.ImageCollection(tail)
                    print("Processing Image Collection "+str(tail))
                    ast=collc.size().getInfo()
                    size=collc.reduceColumns(ee.Reducer.sum(), ['system:asset_size']).get('sum').getInfo()
                    sz=float(size)/1000000
                    unit="MB"
                    with open(output,"a") as csvfile:
                        writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                        writer.writerow([header,tail,ast,format(sz,'.2f'),unit])
                    csvfile.close()
                elif header=="Image":
                    collc=ee.Image(tail)
                    print("Processing Image "+str(tail))
                    size=collc.get('system:asset_size').getInfo()
                    ast="1"
                    sz=float(size)/1000000
                    unit="MB"
                    with open(output,"a") as csvfile:
                        writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                        writer.writerow([header,tail,ast,format(sz,'.2f'),unit])
                    csvfile.close()
                elif header=="Table":
                    collc=ee.FeatureCollection(tail)
                    print("Processing Table "+str(tail))
                    ast="1"
                    size=collc.get('system:asset_size').getInfo()
                    sz=float(size)/1000000
                    unit="MB"
                    with open(output,"a") as csvfile:
                        writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                        writer.writerow([header,tail,ast,format(sz,'.2f'),unit])
                    csvfile.close()
                elif header=="Folder": ##Folders are not added to the list but are print
                    ast=int(b.count(tail))-1
                    print("Folder "+str(tail) +" has "+str(ast)+" assets")
        except Exception:
            return "worked"
    elif items > 0 and typ =='report':
        with open(output,"wb") as csvfile:
            writer=csv.DictWriter(csvfile,fieldnames=["type", "path","No of Assets","size","unit"], delimiter=',')
            writer.writeheader()
            b=subprocess.check_output("earthengine ls -l -r"+'"'+location+'"'+" --max_items "+str(items),shell=True)
        try:
            for item in b.split('\n'):
                a=item.replace("[","").replace("]","").split()
                header=a[0]
                tail=a[1]
                if header=="ImageCollection":
                    collc=ee.ImageCollection(tail)
                    print("Processing Image Collection "+str(tail))
                    ast=collc.size().getInfo()
                    size=collc.reduceColumns(ee.Reducer.sum(), ['system:asset_size']).get('sum').getInfo()
                    sz=float(size)/1000000
                    unit="MB"
                    with open(output,"a") as csvfile:
                        writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                        writer.writerow([header,tail,ast,format(sz,'.2f'),unit])
                    csvfile.close()
                elif header=="Image":
                    collc=ee.Image(tail)
                    print("Processing Image "+str(tail))
                    size=collc.get('system:asset_size').getInfo()
                    ast="1"
                    sz=float(size)/1000000
                    unit="MB"
                    with open(output,"a") as csvfile:
                        writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                        writer.writerow([header,tail,ast,format(sz,'.2f'),unit])
                    csvfile.close()
                elif header=="Table":
                    collc=ee.FeatureCollection(tail)
                    print("Processing Table "+str(tail))
                    ast="1"
                    size=collc.get('system:asset_size').getInfo()
                    sz=float(size)/1000000
                    unit="MB"
                    with open(output,"a") as csvfile:
                        writer=csv.writer(csvfile,delimiter=',',lineterminator='\n')
                        writer.writerow([header,tail,ast,format(sz,'.2f'),unit])
                    csvfile.close()
                elif header=="Folder": ##Folders are not added to the list but are print
                    ast=int(b.count(tail))-1
                    print("Folder "+str(tail) +" has "+str(ast)+" assets")
        except Exception:
            return "worked"
#lst(location="users/samapriya/Belem", typ="report",items=0,output=r"C:\planet_demo\rep.csv")



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
from zipfile import ZipFile

ex = ['.shp', '.prj', '.dbf', '.shx']

file_paths = []
i=1
def zipshape(directory,export):
    for (root, directories, files) in os.walk(directory):
        for filename in files:
            if filename.endswith('.shp'):
                file_paths=[]
                filebase = filename.split('.')[0]
                try:
                    for things in ex:
                        if os.path.exists(os.path.join(root, filebase+things)):
                            filepath = os.path.join(root, filebase+things)
                            #print(filepath)
                            file_paths.append(filepath)
                    os.chdir(export)
                    if int(len(file_paths))==int(4):
                        with ZipFile(filebase+'.zip','w') as zip:
                            print("Creating zipped folder "+str(filebase)+'.zip'+' at '+str(export))
                            # writing each file one by one
                            for file in file_paths:
                                fname=os.path.basename(file)
                                zip.write(file,fname)
                except:
                    pass
#get_all_file_paths(directory=r"C:\Users\samapriya\Downloads\nexgengrid",export=r'D:\Library')



import ee,subprocess,os
##initialize earth engine
ee.Initialize()

suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
def humansize(nbytes):
    i = 0
    while nbytes >= 1024 and i < len(suffixes)-1:
        nbytes /= 1024.
        i += 1
    f = ('%.2f' % nbytes).rstrip('0').rstrip('.')
    return '%s %s' % (f, suffixes[i])
l=[]
def assetsize(asset):
    header=ee.data.getInfo(asset)['type']
    if header=="ImageCollection":
        collc=ee.ImageCollection(asset)
        size=collc.aggregate_array('system:asset_size')
        print('')
        print(str(asset)+" ===> "+str(humansize(sum(size.getInfo()))))
        print('Total number of items in collection: '+str(collc.size().getInfo()))
    elif header=="Image":
        collc=ee.Image(asset)
        print('')
        print(str(asset)+" ===> "+str(humansize(collc.get('system:asset_size').getInfo())))
    elif header=="Table":
        collc=ee.FeatureCollection(asset)
        print('')
        print(str(asset)+" ===> "+str(humansize(collc.get('system:asset_size').getInfo())))
    elif header =="Folder":
        b=subprocess.check_output("earthengine du "+asset+" -s",shell=True)
        num=subprocess.check_output("earthengine ls "+asset,shell=True)
        size=humansize(float(b.strip().split(' ')[0]))
        print('')
        print(str(asset)+" ===> "+str(size))
        print('Total number of items in folder: '+str(len(num.split('\n'))-1))
if __name__ == '__main__':
    assetsize(None)



from __future__ import print_function
import ee
import os
ee.Initialize()

def mover(collection_path,final_path):
    assets_list = ee.data.getList(params={'id': collection_path})
    assets_names = [os.path.basename(asset['id']) for asset in assets_list]
    print('Moving a total of '+str(len(assets_names))+'.....')
    for count,items in enumerate(assets_names):
        print ('Moving '+str(count+1)+' of '+str(len(assets_names)), end='\r')
        init=collection_path+'/'+items
        final=final_path+'/'+items
        try:
            ee.data.renameAsset(init, final)
        except Exception as e:
            pass
#batchcopy(collection_path='users/samapriya/bl',final_path='users/samapriya/b2')




from setuptools import setup
from setuptools import find_packages

setup(
    name='geeadd',
    version='0.2.7',
    packages=['geeadd'],
    package_data={'geeadd': ['logconfig.json']},
    url='https://github.com/samapriya/gee_asset_manager_addon',
    install_requires=['earthengine_api >= 0.1.87','requests >= 2.10.0','poster>=0.8.1','retrying >= 1.3.3','clipboard>=0.0.4','beautifulsoup4 >= 4.5.1','requests_toolbelt >= 0.7.0','pytest >= 3.0.0','future >= 0.16.0','google-cloud-storage >= 1.1.1'],
    license='Apache 2.0',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: GIS',
    ),
    author='Samapriya Roy',
    author_email='samapriya.roy@gmail.com',
    description='Google Earth Engine Batch Assets Manager with Addons',
    entry_points={
        'console_scripts': [
            'geeadd=geeadd.geeadd:main',
        ],
    },
)



import json
import logging.config
import os

default_config = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "simple": {
            "format": "%(asctime)s :: %(name)s :: %(levelname)s :: %(message)s"
        }
    },

    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "level": "DEBUG",
            "formatter": "simple",
            "stream": "ext://sys.stdout"
        },

        "info_file_handler": {
            "class": "logging.handlers.RotatingFileHandler",
            "level": "INFO",
            "formatter": "simple",
            "filename": "gee_assets_info.log",
            "maxBytes": 10485760,
            "backupCount": 20,
            "encoding": "utf8"
        },

        "error_file_handler": {
            "class": "logging.handlers.RotatingFileHandler",
            "level": "ERROR",
            "formatter": "simple",
            "filename": "gee_assets_errors.log",
            "maxBytes": 10485760,
            "backupCount": 20,
            "encoding": "utf8"
        }
    },

    "root": {
        "level": "INFO",
        "handlers": ["console", "info_file_handler", "error_file_handler"]
    }
}

def setup_logging():
    path = os.path.join(os.path.dirname(__file__), 'logconfig.json')
    try:
        with open(path, 'rt') as f:
            config = json.load(f)
    except Exception as e:
        logging.exception('Could not load logconfig.json. Loading default logging configuration.')
        config = default_config
    logging.config.dictConfig(config)



from __future__ import print_function
import ast
import csv
import getpass
import glob
import logging
import os
import sys
import time

if sys.version_info > (3, 0):
    from urllib.parse import unquote
else:
    from urllib import unquote

import ee
import requests
import retrying
import poster

from poster.encode import multipart_encode
from poster.streaminghttp import register_openers
from bs4 import BeautifulSoup

from google.cloud import storage

from metadata_loader import load_metadata_from_csv, validate_metadata_from_csv


def upload(user, source_path, destination_path, metadata_path=None, multipart_upload=False, nodata_value=None, bucket_name=None, band_names=[]):
    """
    Uploads content of a given directory to GEE. The function first uploads an asset to Google Cloud Storage (GCS)
    and then uses ee.data.startIngestion to put it into GEE, Due to GCS intermediate step, users is asked for
    Google's account name and password.

    In case any exception happens during the upload, the function will repeat the call a given number of times, after
    which the error will be propagated further.

    :param user: name of a Google account
    :param source_path: path to a directory
    :param destination_path: where to upload (absolute path)
    :param metadata_path: (optional) path to file with metadata
    :param multipart_upload: (optional) alternative mode op upload - use if the other one fails
    :param nodata_value: (optinal) value to burn into raster for missind data in the image
    :return:
    """
    submitted_tasks_id = {}

    __verify_path_for_upload(destination_path)

    path = os.path.join(os.path.expanduser(source_path), '*.tif')
    all_images_paths = glob.glob(path)

    if len(all_images_paths) == 0:
        logging.error('%s does not contain any tif images.', path)
        sys.exit(1)

    metadata = load_metadata_from_csv(metadata_path) if metadata_path else None

    if user is not None:
        password = getpass.getpass()
        google_session = __get_google_auth_session(user, password)
    else:
        storage_client = storage.Client()

    __create_image_collection(destination_path)

    images_for_upload_path = __find_remaining_assets_for_upload(all_images_paths, destination_path)
    no_images = len(images_for_upload_path)

    if no_images == 0:
        logging.error('No images found that match %s. Exiting...', path)
        sys.exit(1)

    failed_asset_writer = FailedAssetsWriter()

    for current_image_no, image_path in enumerate(images_for_upload_path):
        logging.info('Processing image %d out of %d: %s', current_image_no+1, no_images, image_path)
        filename = __get_filename_from_path(path=image_path)

        asset_full_path = destination_path + '/' + filename

        if metadata and not filename in metadata:
            logging.warning("No metadata exists for image %s: it will not be ingested", filename)
            failed_asset_writer.writerow([filename, 0, 'Missing metadata'])
            continue

        properties = metadata[filename] if metadata else None

        try:
            if user is not None:
                gsid = __upload_file_gee(session=google_session,
                                                  file_path=image_path,
                                                  use_multipart=multipart_upload)
            else:
                gsid = __upload_file_gcs(storage_client, bucket_name, image_path)

            asset_request = __create_asset_request(asset_full_path, gsid, properties, nodata_value, band_names)

            task_id = __start_ingestion_task(asset_request)
            submitted_tasks_id[task_id] = filename
            __periodic_check(current_image=current_image_no, period=20, tasks=submitted_tasks_id, writer=failed_asset_writer)
        except Exception as e:
            logging.exception('Upload of %s has failed.', filename)
            failed_asset_writer.writerow([filename, 0, str(e)])

    __check_for_failed_tasks_and_report(tasks=submitted_tasks_id, writer=failed_asset_writer)
    failed_asset_writer.close()

def __create_asset_request(asset_full_path, gsid, properties, nodata_value, band_names):
    if band_names:
        band_names = [{'id': name} for name in band_names]

    return {"id": asset_full_path,
        "tilesets": [
            {"sources": [
                {"primaryPath": gsid,
                 "additionalPaths": []
                 }
            ]}
        ],
        "bands": band_names,
        "properties": properties,
        "missingData": {"value": nodata_value}
    }

def __verify_path_for_upload(path):
    folder = path[:path.rfind('/')]
    response = ee.data.getInfo(folder)
    if not response:
        logging.error('%s is not a valid destination. Make sure full path is provided e.g. users/user/nameofcollection '
                      'or projects/myproject/myfolder/newcollection and that you have write access there.', path)
        sys.exit(1)


def __find_remaining_assets_for_upload(path_to_local_assets, path_remote):
    local_assets = [__get_filename_from_path(path) for path in path_to_local_assets]
    if __collection_exist(path_remote):
        remote_assets = __get_asset_names_from_collection(path_remote)
        if len(remote_assets) > 0:
            assets_left_for_upload = set(local_assets) - set(remote_assets)
            if len(assets_left_for_upload) == 0:
                logging.warning('Collection already exists and contains all assets provided for upload. Exiting ...')
                sys.exit(1)

            logging.info('Collection already exists. %d assets left for upload to %s.', len(assets_left_for_upload), path_remote)
            assets_left_for_upload_full_path = [path for path in path_to_local_assets
                                                if __get_filename_from_path(path) in assets_left_for_upload]
            return assets_left_for_upload_full_path

    return path_to_local_assets


def retry_if_ee_error(exception):
    return isinstance(exception, ee.EEException)


@retrying.retry(retry_on_exception=retry_if_ee_error, wait_exponential_multiplier=1000, wait_exponential_max=4000, stop_max_attempt_number=3)
def __start_ingestion_task(asset_request):
    task_id = ee.data.newTaskId(1)[0]
    _ = ee.data.startIngestion(task_id, asset_request)
    return task_id


def __validate_metadata(path_for_upload, metadata_path):
    validation_result = validate_metadata_from_csv(metadata_path)
    keys_in_metadata = {result.keys for result in validation_result}
    images_paths = glob.glob(os.path.join(path_for_upload, '*.tif*'))
    keys_in_data = {__get_filename_from_path(path) for path in images_paths}
    missing_keys = keys_in_data - keys_in_metadata

    if missing_keys:
        logging.warning('%d images does not have a corresponding key in metadata', len(missing_keys))
        print('\n'.join(e for e in missing_keys))
    else:
        logging.info('All images have metadata available')

    if not validation_result.success:
        print('Validation finished with errors. Type "y" to continue, default NO: ')
        choice = input().lower()
        if choice not in ['y', 'yes']:
            logging.info('Application will terminate')
            exit(1)


def __extract_metadata_for_image(filename, metadata):
    if filename in metadata:
        return metadata[filename]
    else:
        logging.warning('Metadata for %s not found', filename)
        return None


def __get_google_auth_session(username, password):
    google_accounts_url = 'https://accounts.google.com'
    authentication_url = 'https://accounts.google.com/ServiceLoginAuth'

    session = requests.session()

    login_html = session.get(google_accounts_url)
    soup_login = BeautifulSoup(login_html.content, 'html.parser').find('form').find_all('input')
    payload = {}
    for u in soup_login:
        if u.has_attr('value'):
            payload[u['name']] = u['value']

    payload['Email'] = username
    payload['Passwd'] = password

    auto = login_html.headers.get('X-Auto-Login')
    follow_up = unquote(unquote(auto)).split('continue=')[-1]

    payload['continue'] = follow_up

    session.post(authentication_url, data=payload)
    return session


def __get_upload_url(session):
    # get url and discard; somehow it does not work for the first time
    _ = session.get('https://ee-api.appspot.com/assets/upload/geturl?')
    r = session.get('https://ee-api.appspot.com/assets/upload/geturl?')
    if r.text.startswith('\n<!DOCTYPE html>'):
        logging.error('Incorrect credentials. Probably. If you are sure the credentials are OK, refresh the authentication token. '
                      'If it did not work report a problem. They might have changed something in the Matrix.')
        sys.exit(1)
    d = ast.literal_eval(r.text)
    return d['url']

@retrying.retry(retry_on_exception=retry_if_ee_error, wait_exponential_multiplier=1000, wait_exponential_max=4000, stop_max_attempt_number=3)
def __upload_file_gee(session, file_path,use_multipart):
        upload_url = __get_upload_url(session)
        class IterableToFileAdapter(object):
            def __init__(self, iterable):
                self.iterator = iter(iterable)
                self.length = iterable.total

            def read(self, size=-1):
                return next(self.iterator, b'')

            def __len__(self):
                return self.length

        # define a helper function simulating the interface of posters multipart_encode()-function
        # but wrapping its generator with the file-like adapter
        def multipart_encode_for_requests(params, boundary=None, cb=None):
            datagen, headers = multipart_encode(params, boundary, cb)
            return IterableToFileAdapter(datagen), headers



        # this is your progress callback
        def progress(param, current, total):
            if not param:
                return

            # check out http://tcd.netinf.eu/doc/classnilib_1_1encode_1_1MultipartParam.html
            # for a complete list of the properties param provides to you
            calc=float(current)/float(total)*100
            print ('Uploading '+str(format(float(calc),'.2f'))+" %", end='\r')

        # generate headers and gata-generator an a requests-compatible format
        # and provide our progress-callback
        datagen, headers = multipart_encode_for_requests({
            "file": open(file_path, 'rb'),
            "composite": "NONE",
        }, cb=progress)

        # use the requests-lib to issue a post-request with out data attached
        resp = session.post(upload_url, data=datagen,headers=headers)
        #print(resp.content)

        gsid = resp.json()[0]
        #print(gsid)
        return gsid

@retrying.retry(retry_on_exception=retry_if_ee_error, wait_exponential_multiplier=1000, wait_exponential_max=4000, stop_max_attempt_number=3)
def __upload_file_gcs(storage_client, bucket_name, image_path):
    bucket = storage_client.get_bucket(bucket_name)
    blob_name = __get_filename_from_path(path=image_path)
    blob = bucket.blob(blob_name)

    blob.upload_from_filename(image_path)

    url = 'gs://' + bucket_name + '/' + blob_name

    return url

def __periodic_check(current_image, period, tasks, writer):
    if (current_image + 1) % period == 0:
        logging.info('Periodic check')
        __check_for_failed_tasks_and_report(tasks=tasks, writer=writer)
        # Time to check how many tasks are running!
        __wait_for_tasks_to_complete(waiting_time=10, no_allowed_tasks_running=20)


def __check_for_failed_tasks_and_report(tasks, writer):
    if len(tasks) == 0:
        return

    statuses = ee.data.getTaskStatus(tasks.keys())

    for status in statuses:
        if status['state'] == 'FAILED':
            task_id = status['id']
            filename = tasks[task_id]
            error_message = status['error_message']
            writer.writerow([filename, task_id, error_message])
            logging.error('Ingestion of image %s has failed with message %s', filename, error_message)

    tasks.clear()


def __get_filename_from_path(path):
    return os.path.splitext(os.path.basename(os.path.normpath(path)))[0]


def __get_number_of_running_tasks():
    return len([task for task in ee.data.getTaskList() if task['state'] == 'RUNNING'])


def __wait_for_tasks_to_complete(waiting_time, no_allowed_tasks_running):
    tasks_running = __get_number_of_running_tasks()
    while tasks_running > no_allowed_tasks_running:
        logging.info('Number of running tasks is %d. Sleeping for %d s until it goes down to %d',
                     tasks_running, waiting_time, no_allowed_tasks_running)
        time.sleep(waiting_time)
        tasks_running = __get_number_of_running_tasks()


def __collection_exist(path):
    return True if ee.data.getInfo(path) else False


def __create_image_collection(full_path_to_collection):
    if __collection_exist(full_path_to_collection):
        logging.warning("Collection %s already exists", full_path_to_collection)
    else:
        ee.data.createAsset({'type': ee.data.ASSET_TYPE_IMAGE_COLL}, full_path_to_collection)
        logging.info('New collection %s created', full_path_to_collection)


def __get_asset_names_from_collection(collection_path):
    assets_list = ee.data.getList(params={'id': collection_path})
    assets_names = [os.path.basename(asset['id']) for asset in assets_list]
    return assets_names


class FailedAssetsWriter(object):

    def __init__(self):
        self.initialized = False

    def writerow(self, row):
        if not self.initialized:
            if sys.version_info > (3, 0):
                self.failed_upload_file = open('failed_upload.csv', 'w')
            else:
                self.failed_upload_file = open('failed_upload.csv', 'wb')
            self.failed_upload_writer = csv.writer(self.failed_upload_file)
            self.failed_upload_writer.writerow(['filename', 'task_id', 'error_msg'])
            self.initialized = True
        self.failed_upload_writer.writerow(row)

    def close(self):
        if self.initialized:
            self.failed_upload_file.close()
            self.initialized = False
